"""Wagtail Live utils"""

import re
from functools import lru_cache
from importlib import import_module

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from wagtail.embeds.oembed_providers import all_providers

SUPPORTED_MIME_TYPES = ["png", "jpeg", "gif"]


def get_live_page_model():
    """Retrieves the live page model specified.

    Returns:
        (LivePageMixin) corresponding to the live page model specified.

    Raises:
        (ImproperlyConfigured) if no live page model is specified or the one
        specified doesn't inherit from wagtail_live.models.LivePageMixin.
    """

    from wagtail_live.models import LivePageMixin

    live_model = getattr(settings, "WAGTAIL_LIVE_PAGE_MODEL", "")
    if not live_model:
        raise ImproperlyConfigured(
            "You haven't specified a live page model in your settings."
        )

    dotted_path, model_name = live_model.rsplit(".", 1)
    module = import_module(dotted_path)
    model = getattr(module, model_name)

    if not issubclass(model, LivePageMixin):
        raise ImproperlyConfigured(
            "The live page model specified doesn't inherit from "
            + "wagtail_live.models.LivePageMixin."
        )

    return model


def get_live_receiver():
    """Retrieves the live receiver chosen.

    Returns:
        (BaseMessageReceiver) corresponding to the live receiver specified if found else None.

    Raises:
        (ImproperlyConfigured) if the receiver specified doesn't inherit from
        wagtail_live.receivers.BaseMessageReceiver.
    """

    from wagtail_live.receivers.base import BaseMessageReceiver

    live_receiver = getattr(settings, "WAGTAIL_LIVE_RECEIVER", "")
    if not live_receiver:
        # Assume live interface is used, in which case no additional setup is needed.
        return

    dotted_path, receiver_name = live_receiver.rsplit(".", 1)
    module = import_module(dotted_path)
    receiver = getattr(module, receiver_name)

    if not issubclass(receiver, BaseMessageReceiver):
        raise ImproperlyConfigured(
            f"The receiver {live_receiver} doesn't inherit from "
            + "wagtail_live.receivers.BaseMessageReceiver."
        )
    return receiver


def get_live_publisher():
    """Retrieves the live receiver chosen.

    Returns:
        (Publisher) corresponding to the live publisher specified.

    Raises:
        (ImproperlyConfigured) if no publisher class is specified.
    """

    live_publisher = getattr(settings, "WAGTAIL_LIVE_PUBLISHER", "")
    if not live_publisher:
        raise ImproperlyConfigured(
            "You haven't specified a publisher class in your settings."
        )

    dotted_path, publisher_name = live_publisher.rsplit(".", 1)
    module = import_module(dotted_path)
    publisher = getattr(module, publisher_name)

    return publisher


def get_polling_timeout():
    """Retrieves the duration for the polling timeout for the long polling technique.
    The user can set this parameter in his settings by doing so:
    WAGTAIL_LIVE_POLLING_TIMEOUT = (duration in seconds)
    Defaults to 60(seconds).

    Returns:
        (int) the duration of the polling timeout if defined else 60.
    """

    return getattr(settings, "WAGTAIL_LIVE_POLLING_TIMEOUT", 60)


def get_polling_interval():
    """Retrieves the duration of the polling interval for the interval polling technique.
    The user can set this parameter in his settings by doing so:
    WAGTAIL_LIVE_POLLING_INTERVAL = (duration in ms)
    Defaults to 3000(ms).

    Returns:
        (int) the duration of the polling interval if defined else 3000.
    """

    return getattr(settings, "WAGTAIL_LIVE_POLLING_INTERVAL", 3000)


@lru_cache(maxsize=None)
def is_embed(text):
    """Checks if a text is a link to embed.

    Args:
        text (str): Text to check

    Returns:
        (bool) True if text corresponds to an embed link False else
    """

    for provider in all_providers:
        for url_pattern in provider.get("urls", []):
            if bool(re.match(url_pattern, text)):
                return True

    return False



from functools import lru_cache

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


@lru_cache(maxsize=1)
def get_telegram_bot_token():
    """Retrieves user's telegram bot token.

    Returns:
        (str) telegram bot token.

    Raises:
        (ImproperlyConfigured) if the telegram bot token isn't specified in settings.
    """

    bot_token = getattr(settings, "TELEGRAM_BOT_TOKEN", "")
    if not bot_token:
        raise ImproperlyConfigured(
            "Specify TELEGRAM_BOT_TOKEN if you intend to use Telegram as input source."
        )
    return bot_token


@lru_cache(maxsize=1)
def get_telegram_webhook_url():
    """Retrieves the webhook url which Telegram sends new updates to.

    Returns:
        (str) a URL.

    Raises:
        (ImproperlyConfigured) if the telegram webhook url isn't specified in settings.
    """

    base_webhook_url = getattr(settings, "TELEGRAM_WEBHOOK_URL", "")
    if not base_webhook_url:
        raise ImproperlyConfigured(
            "Specify TELEGRAM_WEBHOOK_URL if you intend to use Telegram as input source."
        )

    if base_webhook_url.endswith("/"):
        base_webhook_url = base_webhook_url[:-1]

    return (
        f"{base_webhook_url}/wagtail_live/telegram/events/{get_telegram_bot_token()}/"
    )


@lru_cache(maxsize=1)
def get_base_telegram_url():
    """Returns the base URL to use when calling Telegram's API."""

    return f"https://api.telegram.org/bot{get_telegram_bot_token()}/"



class BaseWebsocketPublisher:
    """Base class for publishers using the websocket technique."""

    def __call__(self, sender, channel_id, renders, removals, **kwargs):
        """Listens to the live_page_update signal.

        Args:
            sender (LivePageMixin):
                Sender of the signal
            channel_id (str):
                ID of the channel corresponding to the updated page
            renders (dict):
                Dict containing the new posts and the edited posts of the updated page
            removals (list):
                List containing the id of the deleted posts for the updated page

        Returns:
            (None)
        """

        renders = {
            post.id: post.render(context={"block_id": post.id}) for post in renders
        }
        return self.publish(channel_id=channel_id, renders=renders, removals=removals)

    def publish(self, channel_id, renders, removals):
        """Sends a new update:
        - to the "channel_id group" for django channels
        - to an event bus for separate websocket servers
        - to the websocket client for hosted websocket services

        Args:
            channel_id (str):
                ID of the channel corresponding to the updated page
            renders (dict):
                Dict containing the new posts and the edited posts of the updated page
            removals (list):
                List containing the id of the deleted posts for the updated page

        Returns:
            (None)

        Raises:
            NotImplementedError
        """

        raise NotImplementedError



""" Block types and block constructors are defined in this module."""

from django.utils.timezone import now
from wagtail.core.blocks import (
    BooleanBlock,
    CharBlock,
    DateTimeBlock,
    StreamBlock,
    StructBlock,
    StructValue,
    TextBlock,
)
from wagtail.embeds.blocks import EmbedBlock
from wagtail.images.blocks import ImageChooserBlock


class ContentBlock(StreamBlock):
    """A block that represents a live post content."""

    text = TextBlock(help_text="Text of the message")
    image = ImageChooserBlock(help_text="Image of the message")
    embed = EmbedBlock(help_text="URL of the embed message")


class LivePostBlock(StructBlock):
    """A generic block that maps to a message in a messaging app."""

    message_id = CharBlock(help_text="Message's ID")
    created = DateTimeBlock(help_text="Date and time of message creation")
    modified = DateTimeBlock(
        required=False,
        help_text="Date and time of last update",
        blank=True,
    )
    show = BooleanBlock(
        required=False,
        help_text="Indicates if this message is shown/hidden",
        default=True,
    )
    content = ContentBlock()

    class Meta:
        template = "wagtail_live/blocks/live_post.html"

    def clean(self, value):
        """Update modified field when a block is modified via the admin interface."""

        value["modified"] = now()
        return super().clean(value)


def construct_text_block(text):
    """Helper function to construct a text block for a LivePostBlock content.

    Args:
        text (str): Text to add

    Returns:
        a TextBlock
    """

    return TextBlock().to_python(text)


def construct_image_block(image):
    """Helper function to construct an image block for a LivePostBlock content.

    Args:
        image (pk): Foreign key to the image to add

    Returns:
        an ImageBlock
    """

    return ImageChooserBlock().to_python(image.id)


def construct_embed_block(url):
    """Helper function to construct an embed block for a LivePostBlock content.

    Args:
        url (str): Url of the embed

    Returns:
        an EmbedBlock
    """

    return EmbedBlock().to_python(url)


def construct_live_post_block(message_id, created):
    """Helper function to construct a LivePostBlock .

    Args:
        message_id (str):
            Id of the message to construct a live post for.
        created (DateTime):
            Date and time of message creation.

    Returns:
        a LivePostBlock
    """

    return LivePostBlock().to_python(
        {
            "message_id": message_id,
            "created": created,
        }
    )


def add_block_to_live_post(block_type, block, live_block):
    """Adds a new content block to a live post.
    Args:
        block_type (str):
            Type of the block to add
        block (Block):
            Block to add to the live post.
        live_block (LivePostBlock):
            Live post in which the new block will be added.
    """

    if isinstance(live_block, StructValue):
        live_block["content"].append((block_type, block))
    else:
        live_block.value["content"].append((block_type, block))


def clear_live_post_content(live_post):
    """Clears the content of a live post.
    Args:
        live_post (livePostBlock): Live post which content will be cleared.
    """

    if isinstance(live_post, StructValue):
        live_post["content"].clear()
    else:
        live_post.value["content"].clear()
